@@ -1,7 +1,7 @@
 from __future__ import annotations
 import time
 import numpy as np
-from .config import config, logger
+from .config import logger
 from . import internalTools
 from emlib import iterlib
 import weakref as _weakref
@@ -436,8 +436,8 @@ class SynthGroup(AbstrSynth):
     """
     A SynthGroup is used to control multiple synths created
     to work together. Such multiple synths can be groups of
-     similar synths, as in additive synthesis, or processing
-     chains which work as an unity.
+    similar synths, as in additive synthesis, or processing
+    chains which work as an unity.
 
     Attributes:
         synths (List[AbstrSynth]): the list of synths in this group
